;=============== USB Remout Keyboard ===========================

;
;PB0 - 14  ICP1 (IN IRed)
;PB1 - 15  IN KN1
;PB2 - 16  OUT LEDTEST

;PD0 - 2   RXD
;PD1 - 3   TXD
;PD2 - 4   D+ (INT0)
;PD3 - 5   D- (pull-up)
;PD4 - 6   pull-up
;VCC - 7
;GND - 8,22
;AVCC- 20      

;USB Token values
#define USBPID_SETUP     0x2d
#define USBPID_OUT       0xe1
#define USBPID_IN        0x69
#define USBPID_DATA0     0xc3
#define USBPID_DATA1     0x4b
                                      
#define USBPID_ACK       0xd2
#define USBPID_NAK       0x5a
#define USBPID_STALL     0x1e

; USB
.equ USB_BUFSIZE =11       ;PID, 8 bytes data, 2 bytes CRC

;register names for USB
#define x1      r16
#define x2      r17
#define shift   r18
#define cnt     r19
#define x3      r20
#define x4      r21

.equ EEARH =$22      ; MSB EEPROM  88,168

#define token   x1

#define lo8(x)  LOW(x)
#define hi8(x)  ((x)>>8) ;not HIGH to allow XLINK to make a proper range check

.include "m48def.inc"

; idle = J
; J = (D+ = 0), (D- = 1) or USBOUT = 0x08
; K = (D+ = 1), (D- = 0) or USBOUT = 0x04
.equ USBIN        =PIND
.equ USBOUT       =PORTD
.equ USBDDR       =DDRD
.equ USBPLUS      =2     ;signal D+ on PD2 - INT0
.equ USBMINUS     =3     ;signal D- on PD3 - mount 1.5kOhm pull-up on this pin
.equ USBMASK      =(1<<USBPLUS)|(1<<USBMINUS); mask
.equ USBIDLE      =1<<USBMINUS ; J, D-=1, D+=0

.equ KN1          =1     ; PORTB,KN1
.equ LEDTEST      =2     ; PORTB,LED

.dseg ;   
.org $100
; INT0 USB
usbDeviceAddr:     .byte 1  ;assigned during enumeration, defaults to 0
usbNewDeviceAddr:  .byte 1  ;device ID which shoube set after status phase
usbRxToken:        .byte 1  ;token for data we received; if more than 1 rx
                            ;endpoint: MSb=endpoint
usbCurrentTok:     .byte 1  ;last token received, if more than1 rx endpoint:
                            ;MSb=endpoint
usbTxBuf:          .byte USB_BUFSIZE;data to transmit with next IN, free if
                                   ;usbTxLen contains handshake token
usbTxLen:          .byte 1  ;number of bytes to transmit with next
                            ;IN token or handshake token
usbRxBuf:          .byte (2*USB_BUFSIZE);raw RX buffer
usbRxLen:          .byte 1  ;= 0, number of bytes in usbRxBuf
                            ;0 means free,$80 for flow control
usbInputBufOffset: .byte 1  ;offset in usbRxBuf
InpOffsTMP:        .byte 1  ;  offset in usbRxBuf
usbTxBuf1:         .byte USB_BUFSIZE;TX data for endpoint 1
usbTxLen1:         .byte 1  ;TX count for endpoint 1
;usbTxBuf3:         .byte USB_BUFSIZE;TX data for endpoint 3
;usbTxLen3:         .byte 1  ;TX count for endpoint 3

; TX USB token
ReqBTX:            .byte 1  ;  - 
MemBTX:            .byte 1  ;  
LowStr:            .byte 1  ; ..     ROM
HighStr:           .byte 1  ; .. -------------------
CntBTX:            .byte 1  ;   
LngBTX:            .byte 1  ;    ( DATAx,CRC)
StateUSB:          .byte 1  ;   DATA OUT
Config:            .byte 1  ; Config ( )
Idle:              .byte 1  ; Idle ( )
;LED                .byte 1  ;  Set_Report ( )

; RC5
StateRC5:          .byte 1  ;  
BitCnt:            .byte 1  ;  
OldTMR1L:          .byte 1  ;  . TMR1
OldTMR1H:          .byte 1  ; /
ByteRC5L:          .byte 1  ;  RC5 0xxx.xxxx
ByteRC5H:          .byte 1  ;  RC5 000x.xxxx
OldRC5L:           .byte 1  ;   RC5 Txxx.xxxx
RtcRC5:            .byte 1  ;  
BufKbd:            .byte 8  ;  

; TEST
StateTest:         .byte 1  ; =0 -  
AdrTest:           .byte 1  ;   EEPROM
KbdTest:           .byte 1  ;    EEPROM

RTCKN1:            .byte 1  ;    KN1

FLAGS:             .byte 1  ; 
;  FLAGS
.equ BIT_KN1      =0        ;     KN1
.equ BIT_RC5      =1        ;     BufKbd

;  :
.equ TRC5         =1778     ; .  RC5
.equ Ft           =12       ;  , 
.equ Kd           =8        ; Kd TMR1
.equ Nt           =TRC5*Ft/Kd
.equ Nbit         =14       ; -   
.equ RevTMR1      =(1<<ICNC1)|(1<<CS11) ; $82    TCCR1B

.equ DEAD_TIME    =4        ; ~160ms    RC5
.equ KN1_TIME     =5       ; ~0,2sec   KN1

;********************************************************************
;* Interrupt table
;********************************************************************
.cseg
;--------------------------------------------------------------------
.org 0            ;after reset
     rjmp reset
;--------------------------------------------------------------------
.org INT0addr     ;external interrupt INT0
     rjmp INT0handler
;--------------------------------------------------------------------
;  
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
     nop
.org $1A
reset:
     cli                   ;  
     wdr
     in    R16,MCUSR
     sbrs  R16,WDRF        ; WDT Reset
     rjmp  NotWDT
     clr   R16
     out   MCUSR,R16       ; WDRF ==0
     out   USBDDR,R16      ;  USB
     out   USBOUT,R16      ; PD4 - IN, pull up disable
     clr   R17
     clr   R18
;  ~5,5 
Zad5:
     dec   R16
     nop
     brne  Zad5
     wdr
     dec   R17
     brne  Zad5
     dec   R18
     brne  Zad5
;  WDT  8 
NotWDT:
     clr   R16
     out   MCUSR,R16     ; WDRF ==0
; Write logical one to WDCE and WDE
     ldi   R16,$B9       ; 1011.1001
     sts   WDTCSR,R16
     ldi   R16,$A9       ; 1010.1001  WDCE==0
     sts   WDTCSR,R16    ; WDT 8 sec
;  
    ldi   R16,high(RAMEND)
    out   SPH,R16         ; Set Stack Pointer to top of RAM
    ldi   R16,low(RAMEND)
    out   SPL,R16
    clr   R16
    out   EECR,R16        ; EEPROM
    out   EEARH,R16       ; .. 
;  PORTD
    out   USBOUT,R16      ; Pull UP - disable PORTD - IN
    out   USBDDR,R16
;  PORTC
    out   DDRC,R16        ; PORTC - IN
    ldi   R16,$FF         ; Pull UP - enable
    out   PORTC,R16
;  PORTB
    ldi   R16,(1<<LEDTEST); LEDTEST -OUT
    out   DDRB,R16
    ldi   R16,$FF         ; Pull UP - enable, LEDTEST=1)
;    ldi   R16,$FF^(1<<LEDTEST) ; Pull UP - enable, LEDTEST=0)
    out   PORTB,R16
; 
    ldi   R16,(1<<ACD)|(1<<ACI) ; $90 
    out   ACSR,R16
    clr   R16
    sts   DIDR1,R16
; TMR1
    sts   TCNT1H,R16
    sts   TCNT1L,R16
    sts   TCCR1A,R16      ; OC1A/B - disconnect
    sts   TCCR1C,R16
    sts   TIMSK1,R16      ;  . TMR1
    ldi   R16,RevTMR1     ; 1000.0010 Noise,|_,1/8 CLK
    sts   TCCR1B,R16
    ldi   R16,(1<<ICF1)|(1<<TOV1) ; $21
    out   TIFR1,R16       ;   . TMR1
;   RC5
    clr   R16
    sts   StateRC5,R16
    sts   BitCnt,R16
    sts   RTCKN1,R16
    sts   RtcRC5,R16
    sts   StateTest,R16   ;  
    sts   FLAGS,R16       ; 
    com   R16
    sts   OldRC5L,R16     ; =FF
;    sts   ByteRC5L,R16
;    sts   ByteRC5H,R16    ; /
    rcall NullKbd         ;   BufKbd
    rcall USBReset
;  pull-up (PD4=1)
    ldi   R16,$10         ; 0001.0000 PD4=OUT=1
    out   USBOUT,R16
    out   USBDDR,R16
;  INT0
    ldi   R16,(1<<ISC01)|(1<<ISC00) ; 0000.0011 INT0 _|
    sts   EICRA,R16
    ldi   R16,1<<INT0     ; 0000.0001  INT0
    out   EIMSK,R16
    ldi   R16,1<<INTF0    ; 0000.0001   INT0
    out   EIFR,R16
    nop
    sei                   ; Enable interrupts
;--------------------------------------------------
;  
Main:
;  Reset
    ldi   R17,$40
ChkRST:
    in    R16,USBIN
    andi  R16,USBMASK      ;  DATA+, DATA-
    brne  ChkKN1
    dec   R17
    brne  ChkRST           ;  32
    rcall USBReset
    in    R16,USBIN
    andi  R16,USBMASK      ;  DATA+, DATA-
    breq  PC-2             ;   RESET
ChkKN1:
    sbis  PINB,KN1
    rjmp  ChkTMR1          ;  
    clr   R16
    sts   RTCKN1,R16       ;  
ChkTMR1:
    sbis  TIFR1,TOV1       ;   TMR1
    rjmp  ChkTm00          ;  
    lds   R18,FLAGS
    sbr   R18,(1<<BIT_KN1)|(1<<BIT_RC5) ;  KN1  RC5
    sts   FLAGS,R18
    ldi   R17,(1<<TOV1)    ; $20
    out   TIFR1,R17        ;   TOV1
ChkTm00:
    lds   R18,FLAGS
    sbrs  R18,BIT_KN1      ;  TMR1  KN1
    rjmp  ChkTimeRC5       ;  ,   RC5
    lds   R16,StateRC5
    tst   R16
    breq  ChkTm03          ;  
    cpi   R16,$FF
    brne  ChkTimeRC5       ; 0<StateRC5<FF,    RC5  
ChkTm03:    
    lds   R16,StateTest
    tst   R16
    brne  ChkTm01          ; StateTest<>0
    inc   R16
    rjmp  ChkTm02          ; =0,  =1
ChkTm01:
    cpi   R16,$01
    brne  ChkTimeRC5       ; >1,    RC5 TEST
    clr   R16              ; =1,  =0
ChkTm02:
    cbr   R18,(1<<BIT_KN1) ; 
    lds   R17,RTCKN1
    inc   R17
    brne  PC+2
    com   R17              ; =00,  =FF
    sts   RTCKN1,R17
    cpi   R17,KN1_TIME
    brne  ChkTimeRC5       ;   ,  Time RC5
    sts   StateTest,R16    ;   
    in    R17,PORTB
    sbr   R17,(1<<LEDTEST) ; LED=1, 
    tst   R16
    breq  PC+2             ; StateTest=0
    cbr   R17,(1<<LEDTEST) ; LED=0, 
    out   PORTB,R17
ChkTimeRC5:
    sbrs  R18,BIT_RC5      ;  TMR1  RC5
    rjmp  Main01           ;  
    cbr   R18,(1<<BIT_RC5) ; 
    lds   R16,RtcRC5       ;   RC5
    inc   R16
    brne  PC+2
    com   R16              ; =00,  =FF
    sts   RtcRC5,R16
    cpi   R16,DEAD_TIME
    brne  Main01           ;   
    lds   R16,StateTest
    tst   R16
    brne  Main01           ;  TEST,   
    rcall NullKbd          ;   BufKbd
Main01:
    sts   FLAGS,R18        ;  
    rcall RxRC5            ;   RC5
    lds   R16,StateTest
    tst   R16
    brne  Main02           ;  TEST
    rcall PrpReport        ;   BufKbd . 
    rjmp  Main03
Main02:
    rcall PrpTestRpt       ;    TEST
Main03:
    rcall ParsePaket       ;  USB
    rjmp  Main

; ========================= INT0 =================================
;Software-receiver engine. Strict timing! Don't change unless you can
;preserve timing! interrupt response time: 4 cycles + insn running
;= 7 max if interrupts always enabled. max allowable interrupt latency:
;34 cycles -> max 25 cycles interrupt disable
;max stack usage: [ret(2),YL,SREG,YH,shift,x1,x2,x3,cnt,x4] = 11 bytes
;Numbers in brackets are maximum cycles since SOF
; ================================================================
; :
; usbNewDeviceAddr  -  endpoint  handshake
; usbTxLen          - -   ,  bit4 =1
;                     (handshake token), -  IN -   
; usbTxLen1         - --------------------  endpoint 1
; : 
; usbRxToken        -  token PID OUT, SETUP    
; usbCurrentTok:    - =0 -    
; usbRxLen          - -  ,   =0
; usbInputBufOffset -    RX
; usbRxBuf+USB_BUFSIZE-(usbInputBufOffset) -    RX

; :
; usbTxBuf          -    TX endpoint 0
; usbTxBuf1         -    TX endpoint 1
; usbRxBuf          -    RX

; ================================================================
INT0handler:
;order of registers pushed: YL,SREG[sofError],YH,shift,x1,x2,x3,cnt
    push    YL       ;2 [35] push only what is necessary to sync with edge ASAP
    in      YL, SREG ;1 [37]
    push    YL       ;2 [39]
;----------------------------------------------------------------------------
; Synchronize with sync pattern:
;----------------------------------------------------------------------------
;sync byte (D-) pattern LSb to MSb: 01010100 [1 = idle = J, 0 = K]
;sync up with J to K edge during sync pattern -- use fastest possible loops
;first part has no timeout because it waits for IDLE or SE1 (== disconnected)
waitForJ:
    sbis    USBIN, USBMINUS ;1 [40] wait for D- == 1
    rjmp    waitForJ        ;2
waitForK:
;sampling window of 1/4 bit which meets the spec.
    sbis    USBIN, USBMINUS
    rjmp    foundK
    sbis    USBIN, USBMINUS
    rjmp    foundK
    sbis    USBIN, USBMINUS
    rjmp    foundK
    sbis    USBIN, USBMINUS
    rjmp    foundK
    sbis    USBIN, USBMINUS
    rjmp    foundK
    rjmp    sofError
foundK:
;{3,5} after falling D- edge, average delay: 4 cycles [we want 4 for
;center sampling]. We have 1 bit time for setup purposes, then sample again.
;Numbers in brackets are cycles from center of first sync (double K)
;bit after the instruction
    push    YH                  ;2 [2]
    lds     YL, usbInputBufOffset;2 [4]
    clr     YH                  ;1 [5]
    subi    YL, lo8(-(usbRxBuf));1 [6]
    sbci    YH, hi8(-(usbRxBuf));1 [7]

    sbis    USBIN, USBMINUS ;1 [8] we want two bits K [sample 1 cycle too early]
    rjmp    haveTwoBitsK    ;2 [10]
    pop     YH              ; undo the push from before
    rjmp    waitForK        ; this was not the end of sync, retry
; 
haveTwoBitsK:
;----------------------------------------------------------------------------
; push more registers and initialize values while we sample the first bits:
;----------------------------------------------------------------------------
    push    shift           ;2 [16]
    push    x1              ;2 [12]
    push    x2              ;2 [14]

    in      x1, USBIN       ;1 [17] <-- sample bit 0
    ldi     shift, 0xff     ;1 [18]
    bst     x1, USBMINUS    ;1 [19]
    bld     shift, 0        ;1 [20]
    push    x3              ;2 [22]
    push    cnt             ;2 [24]
    
    in      x2, USBIN       ;1 [25] <-- sample bit 1
    ser     x3              ;1 [26] [inserted init instruction]
    eor     x1, x2          ;1 [27]
    bst     x1, USBMINUS    ;1 [28]
    bld     shift, 1        ;1 [29]
    ldi     cnt, USB_BUFSIZE;1 [30] [inserted init instruction]
    rjmp    rxbit2          ;2 [32]

;----------------------------------------------------------------------------
; Receiver loop (numbers in brackets are cycles within byte after instr)
;----------------------------------------------------------------------------

unstuff0:               ;1 (branch taken)
    andi    x3, ~0x01   ;1 [15]
    mov     x1, x2      ;1 [16] x2 contains last sampled (stuffed) bit
    in      x2, USBIN   ;1 [17] <-- sample bit 1 again
    ori     shift, 0x01 ;1 [18]
    rjmp    didUnstuff0 ;2 [20]

unstuff1:               ;1 (branch taken)
    mov     x2, x1      ;1 [21] x1 contains last sampled (stuffed) bit
    andi    x3, ~0x02   ;1 [22]
    ori     shift, 0x02 ;1 [23]
    nop                 ;1 [24]
    in      x1, USBIN   ;1 [25] <-- sample bit 2 again
    rjmp    didUnstuff1 ;2 [27]

unstuff2:               ;1 (branch taken)
    andi    x3, ~0x04   ;1 [29]
    ori     shift, 0x04 ;1 [30]
    mov     x1, x2      ;1 [31] x2 contains last sampled (stuffed) bit
    nop                 ;1 [32]
    in      x2, USBIN   ;1 [33] <-- sample bit 3
    rjmp    didUnstuff2 ;2 [35]

unstuff3:               ;1 (branch taken)
    in      x2, USBIN   ;1 [34] <-- sample stuffed bit 3 [one cycle too late]
    andi    x3, ~0x08   ;1 [35]
    ori     shift, 0x08 ;1 [36]
    rjmp    didUnstuff3 ;2 [38]

unstuff4:               ;1 (branch taken)
    andi    x3, ~0x10   ;1 [40]
    in      x1, USBIN   ;1 [41] <-- sample stuffed bit 4
    ori     shift, 0x10 ;1 [42]
    rjmp    didUnstuff4 ;2 [44]

unstuff5:               ;1 (branch taken)
    andi    x3, ~0x20   ;1 [48]
    in      x2, USBIN   ;1 [49] <-- sample stuffed bit 5
    ori     shift, 0x20 ;1 [50]
    rjmp    didUnstuff5 ;2 [52]

unstuff6:               ;1 (branch taken)
    andi    x3, ~0x40   ;1 [56]
    in      x1, USBIN   ;1 [57] <-- sample stuffed bit 6
    ori     shift, 0x40 ;1 [58]
    rjmp    didUnstuff6 ;2 [60]

; extra jobs done during bit interval:
; bit 0:    store, clear [SE0 is unreliable here due to bit dribbling in hubs]
; bit 1:    se0 check
; bit 2:    overflow check
; bit 3:    recovery from delay [bit 0 tasks took too long]
; bit 4:    none
; bit 5:    none
; bit 6:    none
; bit 7:    jump, eor
rxLoop:
    eor     x3, shift   ;1 [0] reconstruct:x3 is 0 at bit locations we changed,
    in      x1, USBIN   ;1 [1] <-- sample bit 0                    1 at others
    st      y+, x3      ;2 [3] store data
    ser     x3          ;1 [4]
    nop                 ;1 [5]
    eor     x2, x1      ;1 [6]
    bst     x2, USBMINUS;1 [7]
    bld     shift, 0    ;1 [8]
    in      x2, USBIN   ;1 [9] <-- sample bit 1 (or possibly bit 0 stuffed)
    andi    x2, USBMASK ;1 [10]
    breq    se0         ;1 [11] SE0 check for bit 1
    andi    shift, 0xf9 ;1 [12]
didUnstuff0:
    breq    unstuff0    ;1 [13]
    eor     x1, x2      ;1 [14]
    bst     x1, USBMINUS;1 [15]
    bld     shift, 1    ;1 [16]
rxbit2:
    in      x1, USBIN   ;1 [17] <-- sample bit 2 (or possibly bit 1 stuffed)
    andi    shift, 0xf3 ;1 [18]
    breq    unstuff1    ;1 [19] do remaining work for bit 1
didUnstuff1:
    subi    cnt, 1      ;1 [20]
    brcs    overflow    ;1 [21] loop control
    eor     x2, x1      ;1 [22]
    bst     x2, USBMINUS;1 [23]
    bld     shift, 2    ;1 [24]
    in      x2, USBIN   ;1 [25] <-- sample bit 3 (or possibly bit 2 stuffed)
    andi    shift, 0xe7 ;1 [26]
    breq    unstuff2    ;1 [27]
didUnstuff2:
    eor     x1, x2      ;1 [28]
    bst     x1, USBMINUS;1 [29]
    bld     shift, 3    ;1 [30]
didUnstuff3:
    andi    shift, 0xcf ;1 [31]
    breq    unstuff3    ;1 [32]
    in      x1, USBIN   ;1 [33] <-- sample bit 4
    eor     x2, x1      ;1 [34]
    bst     x2, USBMINUS;1 [35]
    bld     shift, 4    ;1 [36]
didUnstuff4:
    andi    shift, 0x9f ;1 [37]
    breq    unstuff4    ;1 [38]
    rjmp    PC+1        ;2 [40]
    in      x2, USBIN   ;1 [41] <-- sample bit 5
    eor     x1, x2      ;1 [42]
    bst     x1, USBMINUS;1 [43]
    bld     shift, 5    ;1 [44]
didUnstuff5:
    andi    shift, 0x3f ;1 [45]
    breq    unstuff5    ;1 [46]
    rjmp    PC+1        ;2 [48]
    in      x1, USBIN   ;1 [49] <-- sample bit 6
    eor     x2, x1      ;1 [50]
    bst     x2, USBMINUS;1 [51]
    bld     shift, 6    ;1 [52]
didUnstuff6:
    cpi     shift, 0x02 ;1 [53]
    brlo    unstuff6    ;1 [54]
    rjmp    PC+1        ;2 [56]
    in      x2, USBIN   ;1 [57] <-- sample bit 7
    eor     x1, x2      ;1 [58]
    bst     x1, USBMINUS;1 [59]
    bld     shift, 7    ;1 [60]
didUnstuff7:
    cpi     shift, 0x04 ;1 [61]
    brsh    rxLoop      ;2 [63] loop control
unstuff7:
    andi    x3, ~0x80   ;1 [63]
    ori     shift, 0x80 ;1 [64]
    in      x2, USBIN   ;1 [65] <-- sample stuffed bit 7
    nop                 ;1 [66]
    rjmp    didUnstuff7 ;2 [68]
;----------------------------------------------------------------------------
;Processing of received packet (numbers in brackets - cycles after end of SE0)
;----------------------------------------------------------------------------
;This is the only non-error exit point for the software receiver loop
;we don't check any CRCs here because there is no time left.
se0:                            ;  [0]
    subi    cnt, USB_BUFSIZE    ;1 [1]
    neg     cnt                 ;1 [2]
    cpi     cnt, 3              ;1 [3]
    ldi     x2, 1<<INTF0        ;1 [4]
    out     EIFR, x2            ;1 [5] clear pending intr and check flag later.
                                ;      SE0 should be over.
    brlo    doReturn            ;1 [6] this is probably an ACK, NAK
                                ;      or similar packet
    sub     YL, cnt             ;1 [7]
    sbci    YH, 0               ;1 [8]
    ld      token, y            ;2 [10]
    cpi     token, USBPID_DATA0 ;1 [11]
    breq    handleData          ;1 [12]
    cpi     token, USBPID_DATA1 ;1 [13]
    breq    handleData          ;1 [14]
    ldd     x2, y+1             ;2 [16] ADDR and 1 bit endpoint number
    mov     x3, x2              ;1 [17] store for endpoint number
    andi    x2, 0x7f            ;1 [18] x2 is now ADDR
    lds     shift, usbDeviceAddr;2 [20]
    cp      x2, shift           ;1 [21]
overflow:             ; This is a hack: brcs overflow will never have Z flag set
    brne    ignorePacket        ;1 [22] packet for different address
    cpi     token, USBPID_IN    ;1 [23]
    breq    handleIn            ;1 [24]
    cpi     token, USBPID_SETUP ;1 [25]
    breq    handleSetupOrOut    ;1 [26]
    cpi     token, USBPID_OUT   ;1 [27]
    breq    handleSetupOrOut    ;1 [28]
;   rjmp    ignorePacket        ;fallthrough, should not happen anyway.

ignorePacket:
    clr     shift
    sts     usbCurrentTok, shift
doReturn:
    pop     cnt
    pop     x3
    pop     x2
    pop     x1
    pop     shift
    pop     YH
sofError:
    pop     YL
    out     SREG, YL
    pop     YL
    reti

;#if USB_CFG_HAVE_INTRIN_ENDPOINT && USB_CFG_HAVE_INTRIN_ENDPOINT3
;handleIn3:                      ;1 [38] (branch taken)
;    lds     cnt, usbTxLen3      ;2 [40]
;    sbrc    cnt, 4              ;2 [42]
;    rjmp    sendCntAndReti      ;0 43 + 17 = 60 until SOP
;    sts     usbTxLen3, x1       ;2 [44] x1 == USBPID_NAK from above
;    ldi     YL, lo8(usbTxBuf3)  ;1 [45]
;    ldi     YH, hi8(usbTxBuf3)  ;1 [46]
;    rjmp    usbSendAndReti      ;2 [48] + 13 = 61 until SOP (violates
                                 ;                 the spec by 1 cycle)
;#endif

;Setup and Out are followed by a data packet two bit times (16 cycles) after
;the end of SE0. The sync code allows up to 40 cycles delay from the start of
;the sync pattern until the first bit is sampled. That's a total of 56 cycles.
handleSetupOrOut:               ;1 [29] (branch taken)
;#if USB_CFG_IMPLEMENT_FN_WRITEOUT  ;if we have data for second OUT endpoint,
;                                   ;set usbCurrentTok to -1
;    sbrc    x3, 7               ;1 [30] skip if endpoint 0
;    ldi     token, -1           ;1 [31] indicate that this is endpoint 1 OUT
;#endif
    sts     usbCurrentTok, token;2 [33]
    pop     cnt                 ;2 [35]
    pop     x3                  ;2 [37]
    pop     x2                  ;2 [39]
    pop     x1                  ;2 [41]
    pop     shift               ;2 [43]
    pop     YH                  ;2 [45]
    in      YL, EIFR            ;1 [46]
    sbrc    YL, INTF0           ;1 [47] check whether data is already arriving
    rjmp    waitForJ            ;2 [49] save the pops and pushes -- a new
                                ;       interrupt is aready pending
    rjmp    sofError            ;2 not an error, it does the pops and reti

handleData:                     ;1 [15] (branch taken)
    lds     token, usbCurrentTok;2 [17]
    tst     token               ;1 [18]
    breq    doReturn            ;1 [19]
    lds     x2, usbRxLen        ;2 [21]
    tst     x2                  ;1 [22]
    brne    sendNakAndReti      ;1 [23]
; 2006-03-11: The following two lines fix a problem where the device was not
; recognized if usbPoll() was called less frequently than once every 4 ms.
    cpi     cnt, 4              ;1 [24] zero sized data packets are status
                                ;       phase only -- ignore and ack
    brmi    sendAckAndReti      ;1 [25] keep rx buffer clean -- we must
                                ;       not NAK next SETUP
    sts     usbRxLen, cnt       ;2 [27] store received data, swap buffers
    sts     usbRxToken, token   ;2 [29]
    lds     x2, usbInputBufOffset;2 [31] swap buffers
    ldi     cnt, USB_BUFSIZE    ;1 [32]
    sub     cnt, x2             ;1 [33]
    sts     usbInputBufOffset, cnt;2 [35] buffers now swapped
    rjmp    sendAckAndReti      ;2 [37] + 19 = 56 until SOP

handleIn:                       ;1 [25] (branch taken)
;We don't send any data as long as the program has not processed the current
;input data and potentially updated the output data. That's more efficient
;in terms of code size than clearing the tx buffers when a packet is received
    lds     x1, usbRxLen        ;2 [27]
    cpi     x1, 1               ;1 [28] negative values are flow control,
                                ;       0 means "buffer free"
    brge    sendNakAndReti      ;1 [29] unprocessed input packet?
    ldi     x1, USBPID_NAK      ;1 [30] prepare value for usbTxLen
;#if USB_CFG_HAVE_INTRIN_ENDPOINT
    sbrc    x3, 7               ;2 [33] x3 contains addr + endpoint
    rjmp    handleIn1           ;0
;#endif
    lds     cnt, usbTxLen       ;2 [34]
    sbrc    cnt, 4              ;2 [36] all handshake tokens have bit 4 set
    rjmp    sendCntAndReti      ;0 37 + 17 = 54 until SOP
    sts     usbTxLen, x1        ;2 [38] x1 == USBPID_NAK from above
    ldi     YL, lo8(usbTxBuf)   ;1 [39]
    ldi     YH, hi8(usbTxBuf)   ;1 [40]
    rjmp    usbSendAndReti      ;2 [42] + 14 = 56 until SOP

; Comment about when to set usbTxLen to USBPID_NAK:
; We should set it back when we receive the ACK from the host. This would
; be simple to implement: One static variable which stores whether the last
; tx was for endpoint 0 or 1 and a compare in the receiver to distinguish the
; ACK. However, we set it back immediately when we send the package,
; assuming that no error occurs and the host sends an ACK. We save one byte
; RAM this way and avoid potential problems with endless retries. The rest of
; the driver assumes error-free transfers anyway.

;#if USB_CFG_HAVE_INTRIN_ENDPOINT /* placed here due to relative jump range */
handleIn1:                      ;1 [33] (branch taken)
;#if USB_CFG_HAVE_INTRIN_ENDPOINT3
; 2006-06-10 as suggested by O.Tamura: support second INTR IN/BULK IN endpoint
;    ldd     x2, y+2             ;2 [35]
;    sbrc    x2, 0               ;2 [37]
;    rjmp    handleIn3           ;0
;#endif
    lds     cnt, usbTxLen1      ;2 [39]
    sbrc    cnt, 4              ;2 [41] all handshake tokens have bit 4 set
    rjmp    sendCntAndReti      ;0 42 + 17 = 59 until SOP
    sts     usbTxLen1, x1       ;2 [43] x1 == USBPID_NAK from above
    ldi     YL, lo8(usbTxBuf1)  ;1 [44]
    ldi     YH, hi8(usbTxBuf1)  ;1 [45]
    rjmp    usbSendAndReti      ;2 [47] + 13 = 60 until SOP
;#endif
;----------------------------------------------------------------------------
; Transmitting data
;----------------------------------------------------------------------------
bitstuff0:                  ;1 (for branch taken)
    eor     x1, x4          ;1
    ldi     x2, 0           ;1
    out     USBOUT, x1      ;1 <-- out
    rjmp    didStuff0       ;2 branch back 2 cycles earlier
bitstuff1:                  ;1 (for branch taken)
    eor     x1, x4          ;1
    rjmp    didStuff1       ;2 we know that C is clear, jump back to do OUT
bitstuff2:                  ;1 (for branch taken)         and ror 0 into x2
    eor     x1, x4          ;1
    rjmp    didStuff2       ;2 jump back 4 cycles earlier and do out and ror
bitstuff3:                  ;1 (for branch taken)         0 into x2
    eor     x1, x4          ;1
    rjmp    didStuff3       ;2 jump back earlier and ror 0 into x2
bitstuff4:                  ;1 (for branch taken)
    eor     x1, x4          ;1
    ldi     x2, 0           ;1
    out     USBOUT, x1      ;1 <-- out
    rjmp    didStuff4       ;2 jump back 2 cycles earlier

sendNakAndReti:                 ;0 [-19] 19 cycles until SOP
    ldi     x3, USBPID_NAK      ;1 [-18]
    rjmp    usbSendX3           ;2 [-16]
sendAckAndReti:                 ;0 [-19] 19 cycles until SOP
    ldi     x3, USBPID_ACK      ;1 [-18]
    rjmp    usbSendX3           ;2 [-16]
sendCntAndReti:                 ;0 [-17] 17 cycles until SOP
    mov     x3, cnt             ;1 [-16]
usbSendX3:                      ;0 [-16]
    ldi     YL, 20              ;1 [-15] 'x3' is R20
    ldi     YH, 0               ;1 [-14]
    ldi     cnt, 2              ;1 [-13]
;   rjmp    usbSendAndReti      fallthrough

; USB spec says:
; idle = J
; J = (D+ = 0), (D- = 1) or USBOUT = 0x04
; K = (D+ = 1), (D- = 0) or USBOUT = 0x02
; Spec allows 7.5 bit times from EOP to SOP for replies (= 60 cycles)

;usbSend:
;pointer to data in 'Y'
;number of bytes in 'cnt' -- including sync byte
;uses: x1...x4, shift, cnt, Y
;Numbers in brackets are time since first bit of sync pattern is sent
usbSendAndReti:             ;0 [-13] timing: 13 cycles until SOP
    in      x2, USBDDR      ;1 [-12]
    ori     x2, USBMASK     ;1 [-11]
    sbi     USBOUT, USBMINUS;2 [-9] prepare idle state; D+ and D- must have been 0 (no pullups)
    in      x1, USBOUT      ;1 [-8] port mirror for tx loop
    out     USBDDR, x2      ;1 [-7] <- acquire bus
; need not init x2 (bitstuff history) because sync starts with 0
    push    x4              ;2 [-5]
    ldi     x4, USBMASK     ;1 [-4] exor mask
    ldi     shift, 0x80     ;1 [-3] sync byte is first byte sent
txLoop:                     ;       [62]
    sbrs    shift, 0        ;1 [-2] [62]
    eor     x1, x4          ;1 [-1] [63]
    out     USBOUT, x1      ;1 [0] <-- out bit 0
    ror     shift           ;1 [1]
    ror     x2              ;1 [2]
didStuff0:
    cpi     x2, 0xfc        ;1 [3]
    brsh    bitstuff0       ;1 [4]
    sbrs    shift, 0        ;1 [5]
    eor     x1, x4          ;1 [6]
    ror     shift           ;1 [7]
didStuff1:
    out     USBOUT, x1      ;1 [8] <-- out bit 1
    ror     x2              ;1 [9]
    cpi     x2, 0xfc        ;1 [10]
    brsh    bitstuff1       ;1 [11]
    sbrs    shift, 0        ;1 [12]
    eor     x1, x4          ;1 [13]
    ror     shift           ;1 [14]
didStuff2:
    ror     x2              ;1 [15]
    out     USBOUT, x1      ;1 [16] <-- out bit 2
    cpi     x2, 0xfc        ;1 [17]
    brsh    bitstuff2       ;1 [18]
    sbrs    shift, 0        ;1 [19]
    eor     x1, x4          ;1 [20]
    ror     shift           ;1 [21]
didStuff3:
    ror     x2              ;1 [22]
    cpi     x2, 0xfc        ;1 [23]
    out     USBOUT, x1      ;1 [24] <-- out bit 3
    brsh    bitstuff3       ;1 [25]
    rjmp    PC+1            ;2 [27]
    ld      x3, y+          ;2 [29]
    sbrs    shift, 0        ;1 [30]
    eor     x1, x4          ;1 [31]
    out     USBOUT, x1      ;1 [32] <-- out bit 4
    ror     shift           ;1 [33]
    ror     x2              ;1 [34]
didStuff4:
    cpi     x2, 0xfc        ;1 [35]
    brsh    bitstuff4       ;1 [36]
    sbrs    shift, 0        ;1 [37]
    eor     x1, x4          ;1 [38]
    ror     shift           ;1 [39]
didStuff5:
    out     USBOUT, x1      ;1 [40] <-- out bit 5
    ror     x2              ;1 [41]
    cpi     x2, 0xfc        ;1 [42]
    brsh    bitstuff5       ;1 [43]
    sbrs    shift, 0        ;1 [44]
    eor     x1, x4          ;1 [45]
    ror     shift           ;1 [46]
didStuff6:
    ror     x2              ;1 [47]
    out     USBOUT, x1      ;1 [48] <-- out bit 6
    cpi     x2, 0xfc        ;1 [49]
    brsh    bitstuff6       ;1 [50]
    sbrs    shift, 0        ;1 [51]
    eor     x1, x4          ;1 [52]
    ror     shift           ;1 [53]
didStuff7:
    ror     x2              ;1 [54]
    cpi     x2, 0xfc        ;1 [55]
    out     USBOUT, x1      ;1 [56] <-- out bit 7
    brsh    bitstuff7       ;1 [57]
    mov     shift, x3       ;1 [58]
    dec     cnt             ;1 [59]
    brne    txLoop          ;1/2 [60/61]
;make SE0:
    cbr     x1, USBMASK     ;1 [61] prepare SE0 [EOP may be 15 to 18 cycles]
    pop     x4              ;2 [63]
;brackets are cycles from start of SE0 now
    out     USBOUT, x1      ;1 [0] <-- out SE0 -- from now 2 bits = 16 cycles
    wdr                     ;1 [1]                             until bus idle
    nop                     ;1 [2]
;    rjmp    PC+1            ;2 [2]
;2006-03-06: moved transfer of new address to usbDeviceAddr
;set address only after data packet was sent, not after handshake
    lds     x2, usbNewDeviceAddr;2 [4]
    subi    YL, 20 + 2      ;1 [5]
    sbci    YH, 0           ;1 [6]
    breq    skipAddrAssign  ;2 [8]
    sts     usbDeviceAddr, x2;0  if not skipped: SE0 is one cycle longer
skipAddrAssign:
;end of usbDeviceAddress transfer
    ldi     x2, 1<<INTF0    ;1 [9] int0 occurred during TX, clear pending flag
    out     EIFR, x2        ;1 [10]
    ori     x1, USBIDLE     ;1 [11]
    in      x2, USBDDR      ;1 [12]
    cbr     x2, USBMASK     ;1 [13] set both pins to input
    mov     x3, x1          ;1 [14]
    cbr     x3, USBMASK     ;1 [15] configure no pullup on both pins
    out     USBOUT, x1      ;1 [16] <-- out J (idle) -- end of SE0 (EOP signal)
    out     USBDDR, x2      ;1 [17] <-- release bus now
    out     USBOUT, x3      ;1 [18] <-- ensure no pull-up resistors are active
    rjmp    doReturn
bitstuff5:                  ;1 (for branch taken)
    eor     x1, x4          ;1
    rjmp    didStuff5       ;2 same trick as above...
bitstuff6:                  ;1 (for branch taken)
    eor     x1, x4          ;1
    rjmp    didStuff6       ;2 same trick as above...
bitstuff7:                  ;1 (for branch taken)
    eor     x1, x4          ;1
    rjmp    didStuff7       ;2 same trick as above...
; ====================  INT0 =====================

; -------------------------------------------------
; ---------------------- / ----------------------
; -------------------------------------------------

; /   IN   TX
; :  BufKbd...BufKbd+7 - 
;        Y -    TX
; : R0=1 -   TX  11 
PrpBufRAM:
   push  YL
   push  YH
   adiw  YL,1             ;  1-    TX
   ldi   XL,low (BufKbd)
   ldi   XH,high (BufKbd)
   ldi   R16,8            ;  
   clr   R0               ;    TX 
PrpBfr02:
   ld    R17,X+           ;     BufKbd
   ld    R18,Y+           ;     TX
   cp    R17,R18
   brne  PrpBfr01         ;  
   dec   R16
   brne  PrpBfr02         ;  .
   pop   YH               ;  ,   
   pop   YL
   ret
PrpBfr01:                 ;  
   pop   YH
   pop   YL
   ld    R16,Y
   cpi   R16,USBPID_DATA0
   brne  PrpBfr04         ;  Data0
   ldi   R16,USBPID_DATA1 ;   Data1
   rjmp  PrpBfr05
PrpBfr04:
   ldi   R16,USBPID_DATA0 ;  Data0
PrpBfr05:
   st    Y+,R16           ; DATA0  DATA1
   push  YL               ;   CRC
   push  YH
   ldi   R16,8            ;  
   ldi   XL,low (BufKbd)
   ldi   XH,high (BufKbd)
PrpBfr03:
   ld    R17,X+
   st    Y+,R17
   dec   R16
   brne  PrpBfr03
   ldi   R18,8            ;   CRC
   pop   YH
   pop   YL
   inc   R0               ;   TX  
   rjmp  usbCrc16         ;    CRC

; /   TX    ROM
; :  LowStr  - ..     ROM
;        HighStr - .. -------------------
;        MemBTX  -  
;        CntBTX  -    TX (  )
;        ReqBTX  -  - 
;        Y       -   TX
; : LngBTX  -    ( DATAx,CRC)
;        R1      - -    TX  ROM
; : R0,R16-R23
PrpBufStr:
    lds   R17,CntBTX
    tst   R17
    brne  PrpBuf01         ;   
;  
    lds   R0,ReqBTX        ;  - 
    lds   R16,MemBTX       ;  
    cp    R0,R16
    brlo  PC+2             ;  <  
    mov   R0,R16           ;>,   
    sts   LngBTX,R0
    ldi   R16,USBPID_DATA1 ;    Data1
    rjmp  PrpBuf05
;  
PrpBuf01:
    ld    R16,Y
    cpi   R16,USBPID_DATA0
    brne  PrpBuf04         ;  Data0
    ldi   R16,USBPID_DATA1 ; Data0   Data1
    rjmp  PrpBuf05
PrpBuf04:
    ldi   R16,USBPID_DATA0 ;  Data0
;   
PrpBuf05:
    st    Y+,R16           ; DATA0  DATA1
    lds   R16,LngBTX
    push  YL               ;      TX
    push  YH               ; /
    tst   R16
    breq  NullPack         ;  
    clr   R0
    cpi   R16,9            ; max   +1
    brlo  PrpBuf02         ; <=8
    subi  R16,8            ; -8
    mov   R0,R16           ;  - 
    ldi   R16,8            ;  8 
PrpBuf02:
    sts   LngBTX,R0        ;  - 
    ldi   R18,3            ; DATAx,CRC
    add   R18,R16          ;    
    mov   R1,R18           ; 
    mov   R18,R16          ; R18= -    ( DATAx,CRC)
    clr   R0
    lds   ZL,LowStr        ;     ROM
    lds   ZH,HighStr
    add   ZL,R17           ;   + 
    adc   ZH,R0            ; /
PrpBuf03:
    lpm
    st    Y+,R0            ;   TX
    adiw  ZL,1             ; .   ROM
    inc   R17              ;   
    dec   R16
    brne  PrpBuf03         ;     TX
    rjmp  PrpCRC
NullPack:
    clr   R17
    sts   MemBTX,R17       ; 0 
    ldi   R18,3
    mov   R1,R18           ; -   
    clr   R18              ;  CRC
PrpCRC:
    sts   CntBTX,R17
    pop   YH               ;   
    pop   YL               ; /
;    rjmp  usbCrc16         ; R18= -   

; /  CRC
; :  Y   -  
;        R18 - 
; : R16 - LOW CRC16
;        R17 - HIGH CRC16
;        (Y+R18),(Y+R18+1) = LOW CRC16, HIGH CRC16
; : R19-R22
usbCrc16:
    ldi     R16,0
    ldi     R17,0
    ldi     R20,lo8(0xa001)
    ldi     R21,hi8(0xa001)
    com     R18       ; R18 = -R18 - 1
crcByteLoop:
    subi    R18, -1
    brcc    crcReady  ; modified loop to ensure that carry is set below
    ld      R22, Y+
    ldi     R19, -8   ; to ensure that carry is set where we need it
    eor     R16,R22
crcBitLoop:
    ror     R17       ; carry is always set here
    ror     R16
    brcs    crcNoXor
    eor     R16,R20
    eor     R17,R21
crcNoXor:
    subi    R19,-1
    brcs    crcBitLoop
    rjmp    crcByteLoop
crcReady:
    st      Y+,R16
    st      Y+,R17
    ret

; /  RC5
; :  StateRC5 -  
;        BitCnt   -  
;        OldTMR1L -  . TMR1
;        OldTMR1H - /
; : Nt=T*Ft/Kd
;            T=1778 - .  RC5
;            Ft=12  -  , 
;            Kd=8   Kd TMR1
; : ByteRC5L -  RC5 0xxx.xxxx
;        ByteRC5H -  RC5 000x.xxxx 
;        StateRC5=FF   
; : R16-R22
RxRC5:
    lds   R16,StateRC5
    cpi   R16,$FF
    brne  PC+2
    ret           ;  , .   .
    cpi   R16,$00
    breq  State0  ; 0
    cpi   R16,$01
    breq  State1  ; 1
    cpi   R16,$02
    breq  State02 ; 2
    cpi   R16,$03
    breq  State03 ; 3
    rjmp  NewRX   ; , 
State02:
    rjmp  State2
State03:
    rjmp  State3
;  1- |_
State0:
    sbic  PINB,0     ; ICP1= PB0=0
    ret              ; =1 - , 
    sbis  TIFR1,ICF1 ; ICF1=1,   |_
    rjmp  NewRX      ; 
;  |_
;  LED
    lds   R18,StateTest
    in    R17,PORTB
    cbr   R17,(1<<LEDTEST) ; LED=0, 
    tst   R18
    breq  PC+2             ; StateTest=0
    sbr   R17,(1<<LEDTEST) ; LED=1, 
    out   PORTB,R17
;
    lds   R17,ICR1L     ;   TMR1
    lds   R18,ICR1H
    sts   OldTMR1L,R17
    sts   OldTMR1H,R18
StateInc:
    inc   R16        ; State+1
    sts   StateRC5,R16
    ret
;     
State1:
    lds   R17,TCNT1L   ;  TMR1
    lds   R18,TCNT1H   ; /
    lds   R19,OldTMR1L
    lds   R20,OldTMR1H
    sub   R17,R19      ; R17,R18 = TMR1-OldTMR1
    sbc   R18,R20
    ldi   R19,LOW(11*Nt/16) ;  11/16 Nt
    ldi   R20,HIGH(11*Nt/16); /
    cp    R17,R19
    cpc   R18,R20
    brsh  PC+2          ;  >11/16 Nt
    ret                 ; 
    clc                 ; C=0
    sbic  PINB,0        ; ICP1= PB0=0
    sec                 ; C=1
    lds   R17,ByteRC5L
    lds   R18,ByteRC5H
    rol   R17           ;  
    rol   R18
    sts   ByteRC5L,R17  ; 
    sts   ByteRC5H,R18  ; /
    com   R17           ; 
    bst   R17,0         ; bit0-->T
    lds   R17,TCCR1B
    bld   R17,ICES1
    sts   TCCR1B,R17    ; bit=0 - _|, =1 - |_
    lds   R17,BitCnt
    inc   R17           ;  
    sts   BitCnt,R17
    ldi   R17,(1<<ICF1) ; $20
    out   TIFR1,R17     ;   ICF1
    rjmp  StateInc      ; . .
State2:
    lds   R17,ICR1L     ;  TMR1
    lds   R18,ICR1H
    lds   R19,OldTMR1L
    lds   R20,OldTMR1H
    sub   R17,R19       ; R17,R18 = ICR1-OldTMR1
    sbc   R18,R20
    ldi   R19,LOW(5*Nt/4) ;  5/4 Nt
    ldi   R20,HIGH(5*Nt/4); /
    cp    R17,R19         ; D(ICR1)-5/4 Nt
    cpc   R18,R20         ; /
    brsh  State2Er        ;  >5/4 Nt, 
    sbis  TIFR1,ICF1      ; ICF1=1
    rjmp  State2Wt        ;     TOUT
    lds   R17,ICR1L       ;  TMR1
    lds   R18,ICR1H
    sts   OldTMR1L,R17    ; 
    sts   OldTMR1H,R18
    lds   R17,BitCnt
    cpi   R17,Nbit-1      ; -  -1
    brsh  PC+3            ;   
    dec   R16             ;  
    rjmp  PC+2
    inc   R16
    sts   StateRC5,R16
;   ICF1,   |_
    lds   R17,TCCR1B
    cbr   R17,(1<<ICES1)  ; bit=0 - |_
    sts   TCCR1B,R17
    ldi   R17,(1<<ICF1)   ; $20
    out   TIFR1,R17       ;   ICF1
    ret
State2Wt:
    lds   R17,TCNT1L   ;  TMR1
    lds   R18,TCNT1H   ; /
    lds   R19,OldTMR1L
    lds   R20,OldTMR1H
    sub   R17,R19      ; R17,R18 = TMR1-OldTMR1
    sbc   R18,R20
    ldi   R19,LOW(3*Nt/2) ;  3/2 Nt
    ldi   R20,HIGH(3*Nt/2); /
    cp    R17,R19
    cpc   R18,R20
State2Er:
    brsh  NewRX         ;  >3/2 Nt, 
    ret                 ; 
;   ,  2Nt
State3:
    lds   R17,TCNT1L
    lds   R18,TCNT1H
    lds   R19,OldTMR1L
    lds   R20,OldTMR1H
    sub   R17,R19         ; TMR1-OldTMR1
    sbc   R18,R20         ; /
    ldi   R19,LOW(2*Nt)
    ldi   R20,HIGH(2*Nt)
    cp    R17,R19
    cpc   R18,R20
    brsh  PC+2            ;  >2 Nt
    ret                   ;  2 Nt
    sbic  TIFR1,ICF1      ; ICF1=0
    rjmp  NewRX           ;   , 
;  -
    lds   R17,ByteRC5L
    lds   R18,ByteRC5H
    rol   R17
    rol   R18
    rol   R17
    rol   R18
    ror   R17
    ror   R17
    set                   ; T=1
    sbrc  R18,6           ; .  =0
    clt                   ; T=0
    bld   R17,6           ;  .
    bst   R18,5           ; tagle bit
    bld   R17,7
    andi  R18,$1F         ; 000x.xxxx
;  
    clr   R16
    sts   RtcRC5,R16      ;  
    lds   R16,OldRC5L
    cp    R17,R16
    breq  NewRX           ;  
    sts   OldRC5L,R17
    andi  R17,$7F         ;  Tagle bit
    sts   ByteRC5L,R17
    sts   ByteRC5H,R18
    ldi   R16,$FF         ;  
    rjmp  PC+2
NewRX:
    clr   R16             ; ,  
    sts   StateRC5,R16
    ldi   R16,RevTMR1     ; 1000.0010
    sts   TCCR1B,R16      ; . TMR1
    ldi   R16,(1<<ICF1)   ; $20
    out   TIFR1,R16       ;   ICF1  .
    clr   R16
    sts   BitCnt,R16      ; . =0
;  LED
    lds   R16,StateTest
    in    R17,PORTB
    sbr   R17,(1<<LEDTEST) ; LED=1, 
    tst   R16
    breq  PC+2             ; StateTest=0
    cbr   R17,(1<<LEDTEST) ; LED=0, 
    out   PORTB,R17
    ret

; /   BufKbd
; EEPROM:
; 00,01 -  
; 02-07 - 
; 08: COM KEY1...(KEY6) FF
; ....................
; :  ByteRC5L -  RC5 0xxx.xxxx
;        ByteRC5H -  RC5 000x.xxxx 
;        StateRC5 =FF    RC5
; : BufKbd...BufKbd+7 -  
;        StateRC5 =00
; : R0,R16-R20,Y
PrpReport:
   lds   R16,StateRC5
   com   R16
   breq  PC+2             ;  
   ret                    ;  
   sts   StateRC5,R16     ;  
;   RC5
DecodeRC5:
   lds   R18,ByteRC5H     ;  RC5 000x.xxxx
   ldi   R16,$02          ;     
DecSisNxt:
   rcall RdEEP            ;    EEPROM
   inc   R16
   eor   R17,R18
   breq  DecSisOK         ;  
   cpi   R16,$08
   brlo  DecSisNxt        ;    07
   ret                    ;   
;  
DecSisOK:
   rcall NullKbd          ;   BufKbd
   ldi   YL,low(BufKbd+2) ;  
   ldi   YH,high(BufKbd+2)
   lds   R18,ByteRC5L     ;  RC5 0xxx.xxxx
   ldi   R16,$08          ;     
SearchCOM:
   rcall RdEEP            ;    EEPROM
   eor   R17,R18
   breq  DecComOK         ;  
;  ,   FF
SearchFF:
   inc   R16
   brne  SearchFF1
   ret                    ;   
SearchFF1:
   rcall RdEEP            ;    EEPROM
   cpi   R17,$FF
   brne  SearchFF         ;  FF
   inc   R16
   brne  SearchCOM        ; ,  
   ret                    ;   
;  
DecComOK:
   ldi   R20,6            ;  
DecComOK1:
   inc   R16
   brne  PC+2             ;   
   ret                    ;   
   rcall RdEEP            ;    EEPROM
   cpi   R17,$FF
   brne  PC+2             ;  FF
   ret                    ;   
   cpi   R17,224
   brlo  NotMod           ; <224
   ldi   R19,8            ; 
   subi  R17,223
;   0   modifier keys
DecMod:
   lds   R0,BufKbd
   ror   R0               ; -->C
DecMod1:
   dec   R17
   brne  PC+2
   sec                    ; C=1
   ror   R0               ;  
   dec   R19
   brne  DecMod1
   sts   BufKbd,R0
   rjmp  DecComOK1        ;  . 
; not modifier key
NotMod:
   st    Y+,R17           ;  
   dec   R20
   brne  DecComOK1        ;  . 
   ret

; /    EEPROM
; :  EEARH=0
;        R16 -.. 
; : R17 - 
RdEEP:
    out   EEARL,R16
    sbi   EECR,EERE       ; Start eeprom read by writing EERE
    in    R17,EEDR        ; Read data from Data Register
    ret

; /    BufKbd
; : R0,R16,Y
NullKbd:
   ldi   YL,low(BufKbd)   ;  
   ldi   YH,high(BufKbd)
   ldi   R16,8
   clr   R0
   st    Y+,R0
   dec   R16
   brne  PC-2
   ret

;=============================================================
;----------------------- / TEST ----------------------------

; /   BufKbd   TEST
; EEPROM:
; 00,01 -  
; 02-07 - 
; 08: COM KEY1...(KEY6) FF
; ....................
; :  ByteRC5L -  RC5 0xxx.xxxx
;        ByteRC5H -  RC5 000x.xxxx 
;        StateRC5 =FF    RC5
;        StateTest =0 -  ,  
;                  =1 - 1-  
;                  =2 -  
;                  =3 - 2-
;                  =4 -  
;                  =5 - 1-  
;                  =6 -  
;                  =7 - 2-
;                  =8 - "="
;                  =9 -      EEPROM
;                  =0A - 1-    
;                  =0B -  
;                  =0C - 2-  
;                  =0D -   09  . .
;                  =0E -  
;                  =0F - 
; : BufKbd+2 -  
;        StateRC5  =0   
;        StateTest =1 ///////////////////////
; : R16-R19,Z
PrpTestRpt:
   lds   R16,StateRC5
   com   R16
   breq  PC+2             ;  
   ret                    ;  
   lds   R16,usbTxLen1
   sbrs  R16,4            ; all handshake tokens have bit 4 set
   ret
;   
   ldi   ZL,low(TableTest) ;    
   ldi   ZH,high(TableTest)
   lds   R16,StateTest
   mov   R18,R16           ;  
   dec   R16               ; -1
   andi  R16,$0F           ; 0000.xxxx
   add   ZL,R16
   clr   R16
   adc   ZH,R16
   ijmp
TableTest:
   rjmp  StTest01
   rjmp  StTest02
   rjmp  StTest03
   rjmp  StTest02
   rjmp  StTest05
   rjmp  StTest02
   rjmp  StTest07
   rjmp  StTest08
   rjmp  StTest09
   rjmp  StTest0A
   rjmp  StTest02
   rjmp  StTest0C
   rjmp  StTest0D
   rjmp  StTest02
   rjmp  StTest0F
   rjmp  StTest0F    ; ,  

StTest01:
   rcall NullKbd          ;   BufKbd
   lds   R16,ByteRC5H
;   andi  R16,$1F          ; 000x.xxxx
   swap  R16
   rjmp  StTestHex
StTest02:
   clr   R16              ;   
   inc   R18
   rjmp  StTestSim
StTest03:
   lds   R16,ByteRC5H
   rjmp  StTestHex
StTest05:
   lds   R16,ByteRC5L
   swap  R16
   rjmp  StTestHex
StTest07:
   lds   R16,ByteRC5L
   rjmp  StTestHex
StTest08:
   ldi   R16,$2E          ; =
   inc   R18
   rjmp  StTestSim
StTest09:
   lds   R19,ByteRC5H     ;  RC5 000x.xxxx
   ldi   R16,$02          ;     
CmpSisNxt:
   rcall RdEEP            ;    EEPROM
   inc   R16
   eor   R17,R19
   breq  CmpSisOK         ;  
   cpi   R16,$08
   brlo  CmpSisNxt        ;    07
   rjmp  StTestEnd        ;   
;  
CmpSisOK:
   ldi   R16,$08          ;     
   lds   R19,ByteRC5L     ;  RC5 Txxx.xxxx
CmpCOM:
   rcall RdEEP            ;    EEPROM
   eor   R17,R19
   breq  CmpComOK         ;  
;  ,   FF
CmpFF:
   inc   R16
   breq  StTestEnd        ;  
   rcall RdEEP            ;    EEPROM
   cpi   R17,$FF
   brne  CmpFF            ;  FF
   inc   R16
   brne  CmpCOM           ; ,  . 
   rjmp  StTestEnd        ;   
;  
CmpComOK:
   inc   R16
   sts   AdrTest,R16      ;   1-  
   ldi   R16,$2C          ; 
   inc   R18
   rjmp  StTestSim
StTest0A:                 ; 1-  
   lds   R16,AdrTest      ; 
   rcall RdEEP            ;    EEPROM  R17
   cpi   R17,$FF
   breq  StTestEnd        ; 
   inc   R16
   breq  StTestEnd
   sts   AdrTest,R16      ; . 
   mov   R16,R17
   sts   KbdTest,R16      ; 
   swap  R16
   rjmp  StTestHex
StTest0C:
   lds   R16,KbdTest      ; 2-  
   rjmp  StTestHex
StTest0D:
   lds   R16,AdrTest      ; 
   rcall RdEEP            ;    EEPROM
   cpi   R17,$FF
   breq  StTestEnd        ; FF
   ldi   R16,$2C          ; 
   ldi   R18,$0A          ;   
   rjmp  StTestSim
StTestEnd:
   ldi   R16,$28          ; Enter
   ldi   R18,$0E          ;   0E
   rjmp  StTestSim
StTest0F:
   clr   R16
   sts   StateRC5,R16     ;   
   ldi   R18,$01
   rjmp  StTestNew

StTestHex:
   inc   R18              ; . 
StTestH01:
   rcall HexAsc
StTestSim:
   sts   (BufKbd+2),R16   ;   
StTestNew:
   sts   StateTest,R18    ;  
   ret

; / HEX-->ASCII
; :  R16 - . = 
; : R16 - ASCII 
HexAsc:
   andi  R16,$0F          ; Make sure we're working with only one nibble
   tst   R16
   breq  HA0              ; =0
   cpi   R16,$0A
   brlo  HA19             ; =1...9.
   subi  R16,$06          ; A=4...F=9
   ret
HA0:
   ldi   R16,$27          ; 0
   ret
HA19:
   subi  R16,$E3          ; (+1D) 1=1E, 9=26
   ret

;=============================================================
;----------------------- / USB -----------------------------
   ; /  USB
USBReset:
    clr   R16
    sts   usbDeviceAddr,R16     ;  =0
    sts   usbNewDeviceAddr,R16
    sts   usbInputBufOffset,R16 ;    RX
    sts   LngBTX,R16            ;     ROM 
    sts   usbRxLen,R16          ;  
    sts   StateUSB,R16          ;    SETUP, IN1
    sts   Config,R16            ; Config ( )
    sts   Idle,R16              ; Idle ( )
    sts   CntBTX,R16            ;    TX
    com   R16                   ; =FF
    sts   usbTxBuf+2,R16        ;  IN (TX)  
    sts   usbTxBuf1+2,R16       ; /
    ldi   R16,USBPID_DATA0
    sts   usbTxBuf,R16
    sts   usbTxBuf1,R16
    ldi   R16,USBPID_NAK
    sts   usbTxLen,R16          ; IN  endpoint 0  
    sts   usbTxLen1,R16         ; IN  endpoint 1  
    ret

; /   USB
; StateUSB =0 -   IN1  SETUP
;          =1 -  IN0,    ROM
;          =3 -  OUT DATA Set_Report
;          =4 -  IN0 Get_Idle
;          =5 -  IN0 Get_Config
;          =6 -  IN0 Get_Report
ParsePaket:
    ldi   YL,LOW(usbRxBuf)     ;     RX
    ldi   YH,HIGH(usbRxBuf)
    adiw  YL,USB_BUFSIZE
    lds   R16,usbInputBufOffset
    sts   InpOffsTMP,R16       ;     RX
    sub   YL,R16
    sbci  YH,0
    ld    R16,Y+               ; PID
    cpi   R16,USBPID_DATA0
    breq  PC+3
    cpi   R16,USBPID_DATA1
    brne  PC+2
    rcall PID_DATA
    lds   R16,StateUSB
    tst   R16
    breq  PID_IN1              ; IN0  
    lds   R16,usbTxLen
    sbrc  R16,4                ; all handshake tokens have bit 4 set
    rcall PID_IN0              ; IN  endpoint 0
;    rjmp  PID_IN1              ; IN  endpoint 1  
;---------------------------------------------------------
; /   IN  endpoint 1
; usbTxBuf1 -    TX endpoint 1
; usbTxLen1 -   =-    usbTxBuf1
PID_IN1:
    ldi   YL,LOW (usbTxBuf1)
    ldi   YH,HIGH (usbTxBuf1)
    rcall PrpBufRAM       ;   TX1 (R0=1 -  )
    ldi   R16,12          ; -  + sync
    tst   R0
    breq  PC+2            ;  ,   
    sts   usbTxLen1,R16
    ret
;---------------------------------------------------------
; /   IN  endpoint 0
; usbTxBuf -    TX endpoint 0
; usbTxLen -   =-    usbTxBuf
; LowStr  - ..     ROM
; HighStr - .. -------------------
; MemBTX  -  
; CntBTX  -    TX (  )
; ReqBTX  -  - 
; StateUSB =1 -  IN0,    ROM
;          =4 -  IN0 Get_Idle
;          =5 -  IN0 Get_Config
;          =6 -  IN0 Get_Report ( )
PID_IN0:
    ldi   YL,LOW (usbTxBuf)
    ldi   YH,HIGH (usbTxBuf)
    lds   R16,StateUSB
    cpi   R16,$01
    brne  InRAM          ;    ROM
    rcall PrpBufStr      ;   token
    mov   R16,R1         ; - 
    rjmp  InEnd          ;    
InRAM:
    cpi   R16,$04
    breq  InIdle
    cpi   R16,$05
    breq  InConfig
;    cpi   R16,$06
;    breq  InReport       ;  report
    rjmp  DoStall        ; 
;InReport:
;    rcall PrpBufRAM      ;   TX (R0=1 -  )
;    ldi   R16,USBPID_NAK
;    tst   R0
;    breq  InEnd1         ;  
;    ldi   R16,12         ; - 
;    rjmp  InEnd          ;    
InIdle:
    lds   R17,Idle
    rjmp  InConf01
InConfig:
    lds   R17,Config
InConf01:
    ldi   R16,USBPID_DATA1
    st    Y+,R16           ;  DATA1
    st    Y,R17            ; Idle  Config
    ldi   R18,1            ; - 
    rcall usbCrc16         ;  CRC
    ldi   R16,4            ; 4   
InEnd:
    inc   R16              ; + sync
INEnd1:
    sts   usbTxLen,R16
    ret

;---------------------------------------------------------
; /    OUT,SETUP   usbRxBuf
; usbRxToken -  token PID OUT,SETUP    
; Y          -   
; usbRxLen   - - ,    =0
PID_DATA:
    lds   R16,usbRxLen
    andi  R16,$7F
    brne  PC+2
    ret                  ;   
    lds   R16,usbInputBufOffset
    lds   R17,InpOffsTMP
    cpse  R16,R17
    ret                  ;  ,   
    lds   R16,usbRxToken
    cpi   R16,USBPID_SETUP
    breq  DoSetup
    rjmp  StCn            ;   DATA0
;    cpi   R16,USBPID_OUT
;    breq  DoOut
;    rjmp  DoStall

;============================================================
; /  PID OUT
; Y        -   
; usbRxLen - - ,    =0
; StateUSB =3 -  OUT DATA Set_Report ( )
;DoOut:
;    rjmp  StCn

; /  PID SETUP
; Y        -   
; usbRxLen - - ,    =0
DoSetup:
    ld    R16,Y+          ; 1-    
    ld    R17,Y+          ; 2- --------------------
    cpi   R16,$00
    breq  Setup_00
    cpi   R16,$21
    breq  Setup_21
    cpi   R16,$A1
    breq  Setup_A1
    cpi   R16,$80
    breq  Setup_80
    cpi   R16,$81
    breq  Setup_81
    cpi   R16,$82
    breq  Setup_82
    rjmp  DoStall         ; 

Setup_00:
    cpi   R17,$05
    breq  SetAdr
    cpi   R17,$09
    breq  SetConfig
    rjmp  DoStall         ; 

Setup_21:
    cpi   R17,$09
    breq  SetReport
    cpi   R17,$0A
    breq  SetIdle
    rjmp  DoStall         ; 

Setup_A1:
;    cpi   R17,$01
;    breq  GetReport
    cpi   R17,$02
    breq  GetIdle
    rjmp  DoStall         ; 

Setup_80:
    cpi   R17,$00
    breq  GetStatDev
    cpi   R17,$06
    breq  GetDescrDev
    cpi   R17,$08
    breq  GetConfig
    rjmp  DoStall         ; 

Setup_81:
    cpi   R17,$00
    breq  GetStatInter
    cpi   R17,$06
    breq  GetDescrInter
    rjmp  DoStall         ; 

Setup_82:
    cpi   R17,$00
    breq  GetStatPoint
    rjmp  DoStall         ; 
;------------------------------------------------------
;    
; R16 - 1-   
; R17 - 2-   
SetIdle:
    adiw  YL,1
    ld    R16,Y           ; 4- 
    sts   Idle,R16
;   
;    clr   R16
;    sts   StateTest,R16   ;  
;    in    R16,PORTB
;    sbr   R16,(1<<LEDTEST) ; LED=1, 
;    out   PORTB,R16
    clr   R16
SetEndx:
    sts   StateUSB,R16
    clr   R16
    sts   usbRxLen,R16    ;  
    ret

SetAdr:
    ld    R16,Y
    sts   usbNewDeviceAddr,R16
StCn:
    clr   R16
    sts   MemBTX,R16      ;  
    sts   CntBTX,R16      ;    TX
    sts   ReqBTX,R16      ;  - 
    ldi   R16,$01         ;    ROM
    rjmp  DoNak

SetConfig:
    ld    R16,Y           ; 3- 
    sts   Config,R16
    rjmp  StCn

SetReport:
    ldi   R16,$03         ;   Report
    rjmp  SetEndx

GetIdle:
    ldi   R16,$04
    rjmp  DoNak

GetConfig:
    ldi   R16,$05
    rjmp  DoNak

;GetReport:
;    ldi   R16,$06         ;    RAM
;    rjmp  DoNak

GetStatDev:                ;   Status=0000
GetStatPoint:              ; /
GetStatInter:              ; /
    ldi   ZL,low(Str0000<<1) ;    0000
    ldi   ZH,high(Str0000<<1)
    sts   LowStr,ZL
    sts   HighStr,ZH
    ldi   R16,2             ;  
    mov   R0,R16
    rjmp  DesAll01

; Get_Descriptor
GetDescrDev:               ;  
GetDescrInter:             ; /
    ld    R18,Y+           ; 3-    (Index)
    ld    R17,Y+           ; 4- (Type)
    cpi   R17,$01
    brne  DesNotDev        ;  Device
; Device_Descriptor
    ldi   ZL,low(DeviceDescr<<1) ;   
    ldi   ZH,high(DeviceDescr<<1)
    rjmp  DesAll

DesNotDev:
    cpi   R17,$02
    brne  DesNotConf       ;  Config
; Config_Descriptor
    ldi   ZL,low(ConfigDescr<<1) ;   
    ldi   ZH,high(ConfigDescr<<1)
    sts   LowStr,ZL
    sts   HighStr,ZH
    adiw  ZL,2             ;   
    rjmp  DesAll02

DesNotConf:
    cpi   R17,$22
    brne  DesNotReport     ;  Report
; Report_Descriptor
    ldi   ZL,low(ReportDescr<<1) ;   
    ldi   ZH,high(ReportDescr<<1)
    sts   LowStr,ZL
    sts   HighStr,ZH
    ldi   R16,LngDesReport ;   
    mov   R0,R16
    rjmp  DesAll01
DesNotReport:
    cpi   R17,$03
    brne  DoStall          ;  String
; String_Descriptor
    cpi   R18,$00
    brne  StrNot0
    ldi   ZL,low(Str0Descr<<1) ;   
    ldi   ZH,high(Str0Descr<<1)
    rjmp  DesAll
StrNot0:
    cpi   R18,$01
    brne  StrNot1
    ldi   ZL,low(Str1Descr<<1) ;   
    ldi   ZH,high(Str1Descr<<1)
    rjmp  DesAll

StrNot1:
    cpi   R18,$02
    brne  DoStall       ;StrNot2
    ldi   ZL,low(Str2Descr<<1) ;   
    ldi   ZH,high(Str2Descr<<1)
;    rjmp  DesAll

;StrNot2:
;    cpi   R18,$03
;    brne  DoStall
;    ldi   ZL,low(Str3Descr<<1) ;   
;    ldi   ZH,high(Str3Descr<<1)
;    rjmp  DesAll

;   .
DesAll:
    sts   LowStr,ZL
    sts   HighStr,ZH
DesAll02:
    lpm
DesAll01:
    sts   MemBTX,R0        ;  
    adiw  YL,2             ;   7- 
    ld    R16,Y
    sts   ReqBTX,R16       ;  - 
    clr   R16
    sts   CntBTX,R16       ;    TX
    ldi   R16,$01          ;    ROM

DoNak:
    ldi   R17,USBPID_NAK
    sts   UsbTxLen,R17
    rjmp  SetEndx

;  
DoStall:
    ldi   R16,USBPID_STALL
    sts   UsbTxLen,R16
    clr   R16
    rjmp  SetEndx

;----------------------------------------------------
;--------------------- / UART ---------------------
;InitUART:
;    ldi   R16,$18    ; 0001.1000 . RX,TX  .
;    sts   UCSR0B,R16
;    ldi   R16,$06    ; 0000.0110 8N1
;    sts   UCSR0C,R16
;    ldi   R16,$0C    ; =F/(8*BOUD)-1= 12000000/(8*115200)-1=12
;    sts   UBRR0L,R16 ; 115385
;    clr   R16
;    sts   UBRR0H,R16
;    ldi   R16,$62    ; 0110.0010   UART duble speed
;    sts   UCSR0A,R16
;    ret

;TXUART:
; : R16 =     
; : R1
;    lds   R1,UCSR0A
;    sbrs  R1,UDRE0
;    rjmp  TXUART       ;  
;    sts   UDR0,R16
;    ret
;
;RXUART:
; : R16 = 
;        R1,RXC0 =1 -  
;                =0 - 
;    lds   R1,UCSR0A
;    sbrc  R1,RXC0    ;  
;    lds   R16,UDR0
;    ret

;-----------------------------------------------------
;-----------------   --------------------
; Get_Status
Str0000:   .db $00,$00

; Device_Descriptor
DeviceDescr:
   .db $12,$01     ; , = Device
   .db $10,$01     ; Ver USB =1.10
   .db $00,$00     ; Class, Subclass
   .db $00,$08     ; Protocol, Packet size
   .db $C0,$16     ; VID lsb,msb
   .db $DC,$05     ; PID lsb,msb
   .db $01,$00     ; Device release number
   .db $01,$02     ; Index of string descriptor manufacturer, product
   .db $00,$01     ; Index of string descriptor serial number, - config

; Config_Descriptor
ConfigDescr:
   .db $09,$02     ; , = CONFIG
   .db $22,$00     ;  
   .db $01,$01     ; Number of interfaces, Config Value
   .db $00,$80     ; $03,$A0 Index of string descriptor Config, Bus powered,remote wake-up
   .db $32,$09     ; Max power =100ma,  Interf Descr
; Interface Descriptor  
;   .db $09         ;  ( config descr)
   .db $04,$00     ; = INTERFASE,  Interf
   .db $00,$01     ; Number of endpoints
   .db $03,$00     ; Class code, No Subclass
   .db $01,$00     ; Keyboard, No string
; HID_Descriptor
   .db $09,$21     ; , = HID
   .db $01,$01     ; HID class
   .db $00,$01     ; Country code, Number of HID class descriptors
   .db $22,LngDesReport ; 22 = REPORT, LSB Total size of report descriptor
   .db $00,$07     ; MSB,  Endpoint
; Endpoint_Descriptor
;   .db $07         ;  ( HID descr)
   .db $05,$81     ; = ENDPOINT, 1 endpoint IN
   .db $03,$08     ; Interrupt endpoint, LSB Max packet size
   .db $00,$0A     ; MSB, Interval for polling

; Report_Descriptor
.equ LngDesReport =62
ReportDescr:
   .db $05,$01,$09,$06,$A1,$01,$05,$07
   .db $19,$E0,$29,$E7,$15,$00,$25,$01
   .db $75,$01,$95,$08,$81,$02,$95,$01
   .db $75,$08,$81,$01,$95,$03,$75,$01
   .db $05,$08,$19,$01,$29,$03,$91,$02
   .db $95,$05,$75,$01,$91,$01,$95,$06
   .db $75,$08,$26,$FF,$00,$05,$07,$19
   .db $00,$29,$DF,$81,$00,$C0

; String_Descriptor 0
Str0Descr:
   .db $04,$03,$09,$04
; String_Descriptor manufacturer
Str1Descr:
   .db $0E,$03     ; , string
   .db 'R',0,'A',0,'4',0,'N',0,'A',0,'L',0
; String descriptor product
Str2Descr:
   .db $20,$03     ; , string
   .db 'R',0,'e',0,'m',0,'o',0,'t',0,'e',0,' ',0
   .db 'k',0,'e',0,'y',0,'b',0,'o',0,'a',0,'r',0,'d',0
; String descriptor config
;Str3Descr:
